@@ -26,11 +26,13 @@ from translator.tests.base import TestCase
 
 class ToscaHotTranslationTest(TestCase):
 
-    def _test_successful_translation(self, tosca_file, hot_file, params=None):
+    def _test_successful_translation(self, tosca_file, hot_files, params=None):
         if not params:
             params = {}
+        if not isinstance(hot_files, list):
+            hot_files = [hot_files]
         diff = TranslationUtils.compare_tosca_translation_with_hot(tosca_file,
-                                                                   hot_file,
+                                                                   hot_files,
                                                                    params)
         self.assertEqual({}, diff, '<difference> : ' +
                          json.dumps(diff, indent=4, separators=(', ', ': ')))
@@ -235,6 +237,15 @@ class ToscaHotTranslationTest(TestCase):
                   'download_url': 'http://www.software.com/download'}
         self._test_successful_translation(tosca_file, hot_file, params)
 
+    def test_hot_translate_software_component_multiple_hosts(self):
+        tosca_file = '../tests/data/tosca_software_component'\
+            '_multiple_hosts.yaml'
+        hot_file = '../tests/data/hot_output/hot_software_component'\
+            '_multiple_hosts.yaml'
+        params = {'cpus': '1',
+                  'download_url': 'http://www.software.com/download'}
+        self._test_successful_translation(tosca_file, hot_file, params)
+
     def test_hot_translate_web_application(self):
         tosca_file = '../tests/data/tosca_web_application.yaml'
         hot_file = '../tests/data/hot_output/hot_web_application.yaml'
@@ -458,26 +469,28 @@ class ToscaHotTranslationTest(TestCase):
         self._test_successful_translation(tosca_file, hot_file, params)
 
     def test_hot_translate_nfv_sample(self):
-        tosca_file = '../tests/data/test_tosca_nfv_sample.yaml'
-        hot_file = '../tests/data/hot_output/hot_nfv_sample.yaml'
+        tosca_file = '../tests/data/nfv/test_tosca_nfv_sample.yaml'
+        hot_file = '../tests/data/hot_output/nfv/hot_nfv_sample.yaml'
         params = {}
         self._test_successful_translation(tosca_file, hot_file, params)
 
     def test_hot_translate_policy(self):
-        tosca_file = '../tests/data/tosca_policies.yaml'
-        hot_file = '../tests/data/hot_output/hot_policies.yaml'
+        tosca_file = '../tests/data/policies/tosca_policies.yaml'
+        hot_file = '../tests/data/hot_output/policies/hot_policies.yaml'
         params = {}
         self._test_successful_translation(tosca_file, hot_file, params)
 
     def test_hot_script_types(self):
-        tosca_file = '../tests/data/test_tosca_script_types.yaml'
+        tosca_file = '../tests/data/interfaces/test_tosca_script_types.yaml'
         hot_file = '../tests/data/hot_output/hot_script_types.yaml'
         params = {}
         self._test_successful_translation(tosca_file, hot_file, params)
 
     def test_hot_interface_on_compute(self):
-        tosca_file = '../tests/data/test_tosca_interface_on_compute.yaml'
-        hot_file = '../tests/data/hot_output/hot_interface_on_compute.yaml'
+        tosca_file = '../tests/data/interfaces/' \
+                     'test_tosca_interface_on_compute.yaml'
+        hot_file = '../tests/data/hot_output/interfaces/' \
+                   'hot_interface_on_compute.yaml'
         params = {}
         self._test_successful_translation(tosca_file, hot_file, params)
 
@@ -494,10 +507,13 @@ class ToscaHotTranslationTest(TestCase):
         self._test_successful_translation(tosca_file, hot_file, params)
 
     def test_hot_translate_scaling_policy(self):
-        tosca_file = '../tests/data/tosca_autoscaling.yaml'
-        hot_file = '../tests/data/hot_output/hot_autoscaling.yaml'
+        tosca_file = '../tests/data/autoscaling/tosca_autoscaling.yaml'
+        hot_files = [
+            '../tests/data/hot_output/autoscaling/hot_autoscaling.yaml',
+            '../tests/data/hot_output/autoscaling/asg_res.yaml',
+            ]
         params = {}
-        self._test_successful_translation(tosca_file, hot_file, params)
+        self._test_successful_translation(tosca_file, hot_files, params)
 
     def test_translate_unsupported_tosca_type(self):
         tosca_file = '../tests/data/test_tosca_unsupported_type.yaml'
@@ -508,11 +524,22 @@ class ToscaHotTranslationTest(TestCase):
                          'type but translation support is not yet available.')
         tosca = ToscaTemplate(tosca_tpl, params, True)
         err = self.assertRaises(UnsupportedTypeError,
-                                TOSCATranslator(tosca, params).translate)
+                                TOSCATranslator(tosca, params)
+                                .translate)
         self.assertEqual(expected_msg, err.__str__())
 
     def test_hot_translate_cluster_scaling_policy(self):
-        tosca_file = '../tests/data/tosca_cluster_autoscaling.yaml'
-        hot_file = '../tests/data/hot_output/hot_cluster_autoscaling.yaml'
+        tosca_file = '../tests/data/autoscaling/tosca_cluster_autoscaling.yaml'
+        hot_file = '../tests/data/hot_output/autoscaling/' \
+                   'hot_cluster_autoscaling.yaml'
         params = {}
         self._test_successful_translation(tosca_file, hot_file, params)
+
+    def test_hot_translate_nfv_scaling(self):
+        tosca_file = '../tests/data/nfv/test_tosca_nfv_autoscaling.yaml'
+        hot_files = [
+            '../tests/data/hot_output/nfv/hot_tosca_nfv_autoscaling.yaml',
+            '../tests/data/hot_output/nfv/SP1_res.yaml',
+            ]
+        params = {}
+        self._test_successful_translation(tosca_file, hot_files, params)
